'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toString = exports.spring = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _toString = require('./to-string');

Object.defineProperty(exports, 'toString', {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_toString).default;
  }
});

var _stepper3 = require('./stepper');

var _stepper4 = _interopRequireDefault(_stepper3);

var _util = require('./util');

var _props = require('./props');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var presets = {
  noWobble: { stiffness: 170, damping: 26 },
  gentle: { stiffness: 120, damping: 14 },
  wobbly: { stiffness: 180, damping: 12 },
  stiff: { stiffness: 210, damping: 20 }
};

// default spring options.
// damping and precision reflect the values of the `wobbly` preset,
// precision defaults to 3 which should be a good tradeoff between
// animation detail and resulting filesize.
var defaultOptions = {
  stiffness: 180,
  damping: 12,
  precision: 3
};

var spring = exports.spring = function spring(startProps, endProps) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  // define stiffness, damping and precision based on default options
  // and options given in arguments.
  var _Object$assign = Object.assign({}, defaultOptions, options, presets[options.preset] || {}),
      stiffness = _Object$assign.stiffness,
      damping = _Object$assign.damping,
      precision = _Object$assign.precision;

  var animatableProps = (0, _props.getAnimatableProps)(startProps, endProps);
  var startValues = (0, _util.mapValues)(animatableProps, function (_ref) {
    var start = _ref.start;
    return start;
  });
  var endValues = (0, _util.mapValues)(animatableProps, function (_ref2) {
    var end = _ref2.end;
    return end;
  });

  var addUnits = function addUnits(object) {
    return (0, _util.mapValues)(object, function (v, k) {
      return '' + v + animatableProps[k].unit;
    });
  };

  var keyframes = {
    '0%': addUnits(startValues),
    '100%': addUnits(endValues)
  };

  Object.keys(startValues).forEach(function (key) {
    var velocity = 0;
    var value = startValues[key];
    var end = endValues[key];

    for (var i = 1; i < 100; i += 1) {
      var _stepper = (0, _stepper4.default)(0.01, value, velocity, end, stiffness, damping);

      var _stepper2 = _slicedToArray(_stepper, 2);

      value = _stepper2[0];
      velocity = _stepper2[1];

      var percent = i + '%';
      keyframes[percent] = Object.assign(keyframes[percent] || {}, _defineProperty({}, key, '' + +value.toFixed(precision) + animatableProps[key].unit));
    }
  });

  return keyframes;
};

// console.log(spring({
//   left: '10px',
//   right: '20em',
//   foo: 'bar',
//   opacity: 0,
//   rotate: '5deg'
// }, {
//   left: '20px',
//   right: '30em',
//   baz: true,
//   opacity: 1,
//   rotate: '10deg'
// }, {
//   preset: 'noWobble'
// }))

exports.default = spring;